/*
    HistoryWindow

	Copyright (c) 2009 by Alexander Kazarin <boiler@co.ru>

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************
*/

#include "historywindow.h"
#include <QPoint>
#include <QApplication>
#include <QDesktopWidget>
#include <QFileInfo>
#include <QDir>
#include "sqlhistory.h"
#include <qutim/iconmanagerinterface.h>
#include <QScrollBar>
#include "myitemdelegat.h"
#include "historymessages.h"


namespace SqlHistoryNamespace {

HistoryWindow::HistoryWindow(const TreeModelItem &item, SqlEngine *engine, QWidget *parent)
	: QWidget(parent), m_item(item), m_engine(engine)
{
    m_collate="";
    if(engine->db.driverName().toLower()=="qmysql") m_collate="CHARACTER SET utf8 COLLATE utf8_unicode_ci";
    ui.setupUi(this);
	
 ui.label_in->setText(tr("").arg(0));
 ui.label_out->setText(tr(""));
 ui.label_all->setText(tr("Records: %L1").arg(0));

 ui.historyPrev->document()->addResource(QTextDocument::ImageResource, QUrl("to.png"), QPixmap(":/to", 0, Qt::AutoColor));
 ui.historyPrev->document()->addResource(QTextDocument::ImageResource, QUrl("from.png"), QPixmap(":/from", 0, Qt::AutoColor));

        this->setWindowTitle("History manager");
        this->setWindowIcon(QIcon(":/history"));
        ui.searchButton->setIcon(QIcon(":/hist16"));
	SystemsCity::PluginSystem()->centerizeWidget(this);
	setAttribute(Qt::WA_QuitOnClose, false);
	setAttribute(Qt::WA_DeleteOnClose, true);
	QList<int> sizes;
	sizes.append(85);
	sizes.append(250);
        //ui.splitter->setSizes(sizes);
        //ui.splitter->setCollapsible(1,false);
	m_history_path = engine->getHistoryPath();
	setIcons();
        initTmpTables();
	fillAccountComboBox();
        fillDirectionComboBox();
	
	connect(ui.accountComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(fillContactComboBox(int)));
	


        ui.dateEdit_to->setDate ( QDate::currentDate () );
        ui.dateEdit_from->setDate ( QDate::currentDate () );


	QStringList account_data; account_data<<m_item.protocol<<m_item.account;
	int account_index = ui.accountComboBox->findData(account_data);

        ui.accountComboBox->setCurrentIndex(account_index);
        fillContactComboBox(account_index);

        connect(ui.fromComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(searchChanged()));
        connect(ui.dirComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(searchChanged()));
        connect(ui.searchButton, SIGNAL(pressed()), this, SLOT(searchChanged()));

        //ui.searchButton->setCheckable(1);

	QStringList contact_data;
        contact_data<<m_item.protocol<<m_item.name;
	int from_index = ui.fromComboBox->findData(contact_data);
        ui.fromComboBox->setCurrentIndex(from_index);
        ui.historyView->setModel(&model);
        ui.historyView->setItemDelegate(new MyItemDelegate());

        connect(ui.historyView->selectionModel(), SIGNAL(selectionChanged ( const QItemSelection & , const QItemSelection & )), this, SLOT(updateSelection(const QItemSelection & , const QItemSelection &)));

        searchChanged();


	show();
}

void HistoryWindow::updateSelection(const QItemSelection &selected,
     const QItemSelection &deselected)
 {
     QModelIndex index;
     QModelIndexList items = selected.indexes();
     if(selected.isEmpty()) return;
     changePrev(selected.first().topLeft());

     /*foreach (index, items) {
         QString text = QString("(%1,%2)").arg(index.row()).arg(index.column());
         model.setData(index, text);
     }*/

     /*items = deselected.indexes();

     foreach (index, items)
         model.setData(index, "");*/

 }

void HistoryWindow::initTmpTables()
{
    QSqlQuery query;

    QString createtable_query="drop table if exists tmp_acc";
    query.exec(createtable_query);
    createtable_query="create temporary table tmp_acc (protocol varchar(64), account varchar(64),name varchar(255) ) "+m_collate+";";
    if(!query.exec(createtable_query)) {qDebug()<<"SQL History"<<"TmpTable tmp_acc create error "<<createtable_query; m_engine->ShowNotify("Database check| tmp_acc create error"); return;}
    QSqlQuery query_g("select protocol, account from history group by protocol, account");
    while (query_g.next()) {
            QString protocol_name = query_g.value(0).toString();
            Icon protocol_icon(protocol_name.toLower(), IconInfo::Protocol);
            QString account_name = query_g.value(1).toString();
            qutim_sdk_0_3::TreeModelItem item;
            item.protocol = protocol_name;
            item.account = account_name;
            item.name = account_name;
            item.type = 0;
            QString account_nickname = account_name; //macek: check before accessing!!
            if (SystemsCity::PluginSystem()->getAdditionalInfoAboutContact(item).count() > 0) {
                    QStringList account_info = SystemsCity::PluginSystem()->getAdditionalInfoAboutContact(item);
                    if (account_info.count()>0) account_nickname = account_info.at(0);
            }
            createtable_query="insert into tmp_acc (protocol, account ,name ) values('"+protocol_name+"','"+account_name+"','"+account_nickname+"')";
            if(!query.exec(createtable_query)) {qDebug()<<"SQL History"<<"TmpTable insert create error "<<createtable_query; m_engine->ShowNotify("Database check| tmp_acc insert error"); return;}

    }
}


void HistoryWindow::initTmpTablesCnt(int index)
{
    QString protocol;
    QString account;
    QString varSql;


    if(index>0)
    {
        QStringList account_data = ui.accountComboBox->itemData(index).toStringList();
        protocol = account_data.at(0);
        account = account_data.at(1);
        varSql=" where protocol='"+protocol+"' and account='"+account+"'";
    }
    else
    {
        varSql = "";
    }

    QSqlQuery query;
    QString createtable_query="drop table if exists tmp_contact";
    query.exec(createtable_query);
    createtable_query="create temporary table tmp_contact (protocol varchar(64), account varchar(64),name varchar(255) ) "+m_collate+";";
    if(!query.exec(createtable_query)) {qDebug()<<"SQL History"<<"TmpTable tmp_contact create error "<<createtable_query; m_engine->ShowNotify("Database check| tmp_contact create error"); return;}

    QSqlQuery query_g("select distinct(itemname),account,protocol from history"+varSql+";");
    while (query_g.next()) {
        QString item_name = query_g.value(0).toString();
        account=query_g.value(1).toString();
        protocol=query_g.value(2).toString();
        Icon protocol_icon(protocol.toLower(), IconInfo::Protocol);
        qutim_sdk_0_3::TreeModelItem item;
        item.protocol = protocol;
        item.account = account;
        item.name = item_name;
        item.type = 0;
        QString contact_name = item_name; //macek: check before accessing!!
        if (SystemsCity::PluginSystem()->getAdditionalInfoAboutContact(item).count() > 0) {
                QStringList contact_info = SystemsCity::PluginSystem()->getAdditionalInfoAboutContact(item);
                if (contact_info.count()>0) contact_name = contact_info.at(0);
        }
            createtable_query="insert into tmp_contact (protocol, account ,name ) values('"+protocol+"','"+item_name+"','"+contact_name+"')";
            if(!query.exec(createtable_query)) {qDebug()<<"SQL History"<<"TmpTable tmp_contact insert error "<<createtable_query; m_engine->ShowNotify("Database check| tmp_contact insert error"); return;}

    }

}
//its ok
void HistoryWindow::setIcons()
{
        //setWindowIcon(Icon("history"));
        //ui.searchButton->setIcon(Icon("search"));
}

//Its ok
void HistoryWindow::fillAccountComboBox()
{
	ui.accountComboBox->clear();
        QSqlQuery query("select protocol, account, name from tmp_acc;");
	while (query.next()) {
		QString protocol_name = query.value(0).toString();
		Icon protocol_icon(protocol_name.toLower(), IconInfo::Protocol);
		QString account_name = query.value(1).toString();
                QString account_nickname = query.value(2).toString(); //macek: check before accessing!!
		QStringList account_data; account_data<<protocol_name<<account_name;
		ui.accountComboBox->addItem(protocol_icon, account_nickname, account_data);
	}


            if(ui.accountComboBox->count()==0)
            {
                Icon protocol_icon(m_item.protocol.toLower(), IconInfo::Protocol);
                qutim_sdk_0_3::TreeModelItem item;
                item.protocol = m_item.protocol;
                item.account = m_item.account;
                item.name = m_item.account;
                item.type = 0;
                Icon protocol_icon2(m_item.protocol.toLower(), IconInfo::Protocol);
                QString account_nickname = m_item.account; //macek: check before accessing!!
                if (SystemsCity::PluginSystem()->getAdditionalInfoAboutContact(item).count() > 0) {
                        QStringList account_info = SystemsCity::PluginSystem()->getAdditionalInfoAboutContact(item);
                        if (account_info.count()>0) account_nickname = account_info.at(0);
                }
                QStringList account_data; account_data<<m_item.protocol<<m_item.account;
                ui.accountComboBox->addItem(protocol_icon2, account_nickname, account_data);

            }
            ui.accountComboBox->insertItem(0, "All accounts", "");
            //if(ui.accountComboBox->count()==1)ui.accountComboBox->setCurrentIndex(0);
}

//Setup direction (may be make it from SQL?)
void HistoryWindow::fillDirectionComboBox()
{
        ui.dirComboBox->clear();
        ui.dirComboBox->addItem("Both", "");
        ui.dirComboBox->addItem("In", "direction=1");
        ui.dirComboBox->addItem("Out", "direction=0");

}

//Its ok
void HistoryWindow::fillContactComboBox(int index)
{
    QString protocol;
    QString account;
    QString varSql;
	ui.fromComboBox->clear();

        if (index>-1) {
                initTmpTablesCnt(index);
                QSqlQuery query("select protocol, account, name from tmp_contact;");
		while (query.next()) {
                        QString contact_name = query.value(2).toString();
                        account=query.value(1).toString();
                        protocol=query.value(0).toString();
			Icon protocol_icon(protocol.toLower(), IconInfo::Protocol);
                        QStringList item_data; item_data<<protocol<<account;
			ui.fromComboBox->addItem(protocol_icon, contact_name, item_data);
		}
	}


        ui.fromComboBox->insertItem(0,"All contacts", "");

        QStringList contact_data;
        contact_data<<m_item.protocol<<m_item.name;
        int from_index = ui.fromComboBox->findData(contact_data);
        //ui.fromComboBox->setCurrentIndex(from_index);

       if(ui.accountComboBox->currentIndex()>0)
        {
            QStringList account_data = ui.accountComboBox->itemData(ui.accountComboBox->currentIndex()).toStringList();
            protocol = account_data.at(0);

            if((protocol.toLower()==m_item.protocol.toLower()) && (from_index==-1) && (m_item.name.length()>0))
            {
                Icon protocol_icon(m_item.protocol.toLower(), IconInfo::Protocol);
                qutim_sdk_0_3::TreeModelItem item;
                item.protocol = m_item.protocol;
                item.account = m_item.account;
                item.name = m_item.name;
                item.type = 0;
                QString contact_name =  m_item.name; //macek: check before accessing!!
                if (SystemsCity::PluginSystem()->getAdditionalInfoAboutContact(item).count() > 0) {
                        QStringList contact_info = SystemsCity::PluginSystem()->getAdditionalInfoAboutContact(item);
                        if (contact_info.count()>0) contact_name = contact_info.at(0);
                }
                QStringList item_data; item_data<<m_item.protocol<<m_item.name;
                ui.fromComboBox->insertItem(1,protocol_icon,contact_name, item_data);
            }

        }

}


void HistoryWindow::searchChanged( )
{
    QString history_html;
    QString protocol="";
    QString account="";
    QString itemname="";
    QString varSql="";
    QString account_nickname,contact_nickname; //macek: check before accessing!!

    int in=0, out=0;
    m_search_word = ui.searchEdit->text();
    QTime fromt(0, 0, 0);
    QTime tot(23, 59, 59);
    ui.dateEdit_from->setTime(fromt);
    ui.dateEdit_to->setTime(tot);

    //to-do control for abnormal date
    uint from_date_ts = ui.dateEdit_from->dateTime().toTime_t();
    uint to_date_ts = ui.dateEdit_to->dateTime().toTime_t();

//    uint from_date_ts = QDateTime::fromString(year+"-"+month+"-"+day+" 00:00:00", "yyyy-MM-dd HH:mm:ss").toTime_t();
//    uint to_date_ts = QDateTime::fromString(year+"-"+month+"-"+day+" 23:59:59", "yyyy-MM-dd HH:mm:ss").toTime_t();


    if(ui.accountComboBox->currentIndex()>0)
    {

        QStringList account_data = ui.accountComboBox->itemData ( ui.accountComboBox->currentIndex (),  Qt::UserRole ).toStringList(); ;
        protocol = account_data[0];
        account = account_data[1];
        varSql= " h.protocol='"+protocol+"' and h.account='"+account+"'";
    }
    else
    {
      account_nickname="";
    }


    if(ui.fromComboBox->currentIndex()>0)
    {
       QStringList contact_data = ui.fromComboBox->itemData ( ui.fromComboBox->currentIndex (),  Qt::UserRole ).toStringList(); ;

       protocol = contact_data[0];
       account = contact_data[1];

        if(varSql.length()>0) varSql+=" and";
        varSql+= " h.itemname='"+account+"'";
    }
    else
    {
        contact_nickname = "";
    }

    //date to-do chaeck box for all
    if(!ui.checkBox_from->isChecked())
    {
        if(varSql.length()>0) varSql+=" and";
        varSql=varSql+" h.datetime>="+QString::number(from_date_ts);
    }

    if(!ui.checkBox_to->isChecked())
    {
        if(varSql.length()>0) varSql+=" and";
        varSql=varSql+" h.datetime<="+QString::number(to_date_ts);
    }

    if( ui.dirComboBox->currentIndex()>0)
    {
        varSql+=" and "+ui.dirComboBox->itemData(ui.dirComboBox->currentIndex(),Qt::UserRole).toString();
    }

    QString like_clause = (!m_search_word.isEmpty())? " and h.text like '%" + m_search_word + "%'" : "";

    if(varSql.length()>0) varSql=" where"+varSql;


    model.setQuery("SELECT h.protocol, a.name as accname, h.direction, c.name as contname, h.datetime, h.text, h.id, h.account, h.itemname  FROM history h left join tmp_acc a on (h.account=a.account and h.protocol=a.protocol) left join tmp_contact c on (h.itemname=c.account and h.protocol=c.protocol)"+varSql+like_clause+" order by datetime;");
    model.setHeaderData(0, Qt::Horizontal, QObject::tr(""));
    model.setHeaderData(1, Qt::Horizontal, QObject::tr("Account"));
    model.setHeaderData(2, Qt::Horizontal, QObject::tr(""));
    model.setHeaderData(3, Qt::Horizontal, QObject::tr("Contact"));
    model.setHeaderData(4, Qt::Horizontal, QObject::tr("Date & Time"));
    model.setHeaderData(5, Qt::Horizontal, QObject::tr("Message"));
    ui.historyView->setColumnHidden(6, true);
    ui.historyView->setColumnHidden(7, true);
    ui.historyView->setColumnHidden(8, true);
    ui.historyView->setColumnHidden(1, !((ui.fromComboBox->currentIndex()==0) && (ui.accountComboBox->currentIndex()==0)));
    ui.historyView->setColumnWidth(0,30);
    ui.historyView->setColumnWidth(1,100);
    ui.historyView->setColumnWidth(2,30);
    ui.historyView->setColumnWidth(3,100);
    ui.historyView->setColumnWidth(4,140);
    ui.historyView->setColumnWidth(5,300);

    ui.label_all->setText(tr("Records: %L1").arg(model.rowCount()));
    ui.historyPrev->setHtml("<p align='center'>"
                            + tr("Select row for view message") + "</p>");
}


void HistoryWindow::changePrev(QModelIndex index)
{
    QString history_html;
    if(!index.isValid())return;
    //qDebug()<<index.row();
    QSqlRecord rec=model.record(index.row());

    QDateTime datetime = QDateTime::fromTime_t(rec.field("datetime").value().toUInt());
    QString message = rec.field("text").value().toString();
    QString account_nickname=rec.field("accname").value().toString();
    QString contact_nickname=rec.field("contname").value().toString();
    QString protocol=rec.field("protocol").value().toString();
    uint direction = rec.field("direction").value().toUInt();

    if(account_nickname.isNull() || account_nickname.isEmpty()) account_nickname= rec.field("account").value().toString();
    if(contact_nickname.isNull() || contact_nickname.isEmpty()) contact_nickname= rec.field("itemname").value().toString();

    history_html = "<img src='"+SystemsCity::IconManager()->getIconPath( protocol.toLower(), IconInfo::Protocol)+"'>";

    if((ui.fromComboBox->currentIndex()==0) && (ui.accountComboBox->currentIndex()==0))
    {
         if(direction==0)
         {
             history_html += "<b><font color='red'>"+account_nickname+"</font></b> <img src=\":from.png\"> "+"<b><font color='blue'>"+contact_nickname;
         }
         else
         {
             history_html += "<b><font color='blue'>"+contact_nickname+"</font></b> <img src=\":from.png\"> "+"<b><font color='red'>"+account_nickname;
         }

    }
    else
    {
        history_html += (direction==0) ? "<b><font color='red'>"+account_nickname : "<b><font color='blue'>"+contact_nickname;
    }
        history_html +="</font></b><br/>";

    history_html += "<b><font size='small' color='navy'> ( "+datetime.toString("dd.MM.yyyy hh:mm:ss")+" )</font></b><br />";
    //history_html += "<a id='" + query.value(3).toString()+ "'>";
    int pos = message.indexOf(m_search_word, 0, Qt::CaseInsensitive);
    if (!m_search_word.isEmpty() && (pos != -1)) {
            QString tmp = message.mid(pos, m_search_word.length());
            history_html += message.replace(tmp, "<font style=\"background-color: yellow\">"+tmp+"</font>");
    } else history_html += message;
    history_html += "</a><br /><br />";
    ui.historyPrev->setHtml(history_html);
}


}



void SqlHistoryNamespace::HistoryWindow::on_checkBox_from_stateChanged(int )
{
    ui.dateEdit_from->setEnabled(!ui.checkBox_from->isChecked());
     searchChanged();
}

void SqlHistoryNamespace::HistoryWindow::on_checkBox_to_stateChanged(int )
{
    ui.dateEdit_to->setEnabled(!ui.checkBox_to->isChecked());
     searchChanged();
}

void SqlHistoryNamespace::HistoryWindow::on_dateEdit_from_dateChanged(QDate date)
{
   if(ui.dateEdit_to->date()<date) ui.dateEdit_from->setDate ( ui.dateEdit_to->date());
   searchChanged();
}

void SqlHistoryNamespace::HistoryWindow::on_dateEdit_to_dateChanged(QDate date)
{
     if(ui.dateEdit_from->date()>date) ui.dateEdit_to->setDate ( ui.dateEdit_from->date());
    searchChanged();
}

void SqlHistoryNamespace::HistoryWindow::on_searchEdit_textChanged(QString )
{
    //searchChanged();
}

void SqlHistoryNamespace::HistoryWindow::on_searchEdit_editingFinished()
{
   searchChanged();
}

void SqlHistoryNamespace::HistoryWindow::on_historyView_clicked(QModelIndex index)
{
    on_historyView_activated(index);
}

void SqlHistoryNamespace::HistoryWindow::on_historyView_activated(QModelIndex index)
{

}

void SqlHistoryNamespace::HistoryWindow::on_toolButton_clicked() //1
{
    ui.dateEdit_from->setDate ( QDate::currentDate().addDays(-1));
    ui.dateEdit_to->setDate ( QDate::currentDate());
    //searchChanged();
}

void SqlHistoryNamespace::HistoryWindow::on_toolButton_2_clicked() //7
{
 ui.dateEdit_from->setDate ( QDate::currentDate().addDays(-7));
 ui.dateEdit_to->setDate ( QDate::currentDate());
}

void SqlHistoryNamespace::HistoryWindow::on_toolButton_3_clicked() //14
{
 ui.dateEdit_from->setDate (  QDate::currentDate().addDays(-14));
 ui.dateEdit_to->setDate ( QDate::currentDate());
}

void SqlHistoryNamespace::HistoryWindow::on_toolButton_4_clicked() //30
{
 ui.dateEdit_from->setDate ( QDate::currentDate().addMonths(-1));
 ui.dateEdit_to->setDate ( QDate::currentDate());
}


void SqlHistoryNamespace::HistoryWindow::on_historyView_doubleClicked(QModelIndex index)
{
    if(!index.isValid())return;
    //qDebug()<<index.row();
    QSqlRecord rec=model.record(index.row());

    unsigned int id = rec.field("id").value().toUInt();

    QString history_html;
    QString protocol="";
    QString account="";
    QString itemname="";
    QSqlQuery query("SELECT h.protocol, a.name as accname, h.direction, c.name as contname, h.datetime, h.text, h.id, h.account, h.itemname  FROM history h left join tmp_acc a on (h.account=a.account and h.protocol=a.protocol) left join tmp_contact c on (h.itemname=c.account and h.protocol=c.protocol) where id<="+QString::number(id)
                    +" AND h.account='"+ rec.field("account").value().toString() +"' AND h.itemname='"+rec.field("itemname").value().toString()+"' ORDER BY h.id DESC LIMIT 10;");
        //macek: remove children

   while (query.next());

        while (query.previous()) {
                QDateTime datetime = QDateTime::fromTime_t(query.value(4).toUInt());
                QString message = query.value(5).toString();
                uint direction = query.value(2).toUInt();

                    account=query.value(7).toString();
                    protocol=query.value(0).toString();
                    QString account_nickname = query.value(1).toString();
                    if(account_nickname.isNull() || account_nickname.isEmpty()) account_nickname=account;


                    itemname=query.value(8).toString();
                    QString contact_nickname = query.value(3).toString();
                    if(contact_nickname.isNull() || contact_nickname.isEmpty()) contact_nickname=itemname;
                if(query.value(6).toUInt()==id) history_html += "<div style='background-color: #f3f4f6;'>"; else history_html += "<div>";
                history_html += "<img src='"+SystemsCity::IconManager()->getIconPath( protocol.toLower(), IconInfo::Protocol)+"'>";


                     if(direction==0)
                     {
                        history_html += "<b><font color='red'>"+account_nickname+"</font></b> <img src=\":from.png\"> "+"<b><font color='blue'>"+contact_nickname;
                     }
                     else
                     {
                         history_html += "<b><font color='blue'>"+contact_nickname+"</font></b> <img src=\":from.png\"> "+"<b><font color='red'>"+account_nickname;
                     }


                    history_html +="</font></b><br/>";

                history_html += "<b><font size='small' color='navy'> ( "+datetime.toString("dd.MM.yyyy hh:mm:ss")+" )</font></b><br />";
                int pos = message.indexOf(m_search_word, 0, Qt::CaseInsensitive);
                if (!m_search_word.isEmpty() && (pos != -1)) {
                        QString tmp = message.mid(pos, m_search_word.length());
                        history_html += message.replace(tmp, "<font style=\"background-color: yellow\">"+tmp+"</font>");
                } else history_html += message;
                history_html += "</a><br /><br /></div>";


        }

        query.exec("SELECT h.protocol, a.name as accname, h.direction, c.name as contname, h.datetime, h.text, h.id, h.account, h.itemname  FROM history h left join tmp_acc a on (h.account=a.account and h.protocol=a.protocol) left join tmp_contact c on (h.itemname=c.account and h.protocol=c.protocol) where id>"+QString::number(id)
                        +" AND h.account='"+ rec.field("account").value().toString() +"' AND h.itemname='"+rec.field("itemname").value().toString()+"' ORDER BY h.id LIMIT 10;");
            //macek: remove children

            while (query.next()) {
                    QDateTime datetime = QDateTime::fromTime_t(query.value(4).toUInt());
                    QString message = query.value(5).toString();
                    uint direction = query.value(2).toUInt();

                        account=query.value(7).toString();
                        protocol=query.value(0).toString();
                        QString account_nickname = query.value(1).toString();
                        if(account_nickname.isNull() || account_nickname.isEmpty()) account_nickname=account;


                        itemname=query.value(8).toString();
                        QString contact_nickname = query.value(3).toString();
                        if(contact_nickname.isNull() || contact_nickname.isEmpty()) contact_nickname=itemname;
                    //if(query.value(6).toUInt()==id) history_html += "<div style='background-color: #009ACF;'>"; else history_html += "<div>";
                    history_html += "<div><img src='"+SystemsCity::IconManager()->getIconPath( protocol.toLower(), IconInfo::Protocol)+"'>";


                         if(direction==0)
                         {
                            history_html += "<b><font color='red'>"+account_nickname+"</font></b> <img src=\":from.png\"> "+"<b><font color='blue'>"+contact_nickname;
                         }
                         else
                         {
                             history_html += "<b><font color='blue'>"+contact_nickname+"</font></b> <img src=\":from.png\"> "+"<b><font color='red'>"+account_nickname;
                         }


                        history_html +="</font></b><br/>";

                    history_html += "<b><font size='small' color='navy'> ( "+datetime.toString("dd.MM.yyyy hh:mm:ss")+" )</font></b><br />";
                    int pos = message.indexOf(m_search_word, 0, Qt::CaseInsensitive);
                    if (!m_search_word.isEmpty() && (pos != -1)) {
                            QString tmp = message.mid(pos, m_search_word.length());
                            history_html += message.replace(tmp, "<font style=\"background-color: yellow\">"+tmp+"</font>");
                    } else history_html += message;
                    history_html += "</a><br /><br /></div>";


            }

    //qDebug()<<history_html;
    historymessages *mymsg=new historymessages(history_html,this);
    mymsg->exec();

}
